SystemOrganization addCategory: #AGV!
SystemOrganization addCategory: 'AGV-SIM'!

GeoDSLDrawing subclass: #AgvDSLDrawing
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AGV-SIM'!

!AgvDSLDrawing commentStamp: '<historical>' prior: 0!
An AgvDSLDrawing is a GeoDSLDrawing adding the agv-interface.

It assumes an object named #boundary - the terminal boundary


!

!AgvDSLDrawing methodsFor: 'interface-agv' stamp: 'hk 8/28/2014 14:36'!
boundaryPointsWithin: aRadius of: aPoint 
	"answer all boundary points within aRadius around aPoint.
	 PRE the boundary object is named 'boundary' "
	
	 ^objectDict at: #boundary ifPresent: [:v | v  pointsWithin: aRadius from: aPoint  ] ifAbsent: [ OrderedCollection new ].
	! !

!AgvDSLDrawing methodsFor: 'interface-agv' stamp: 'hk 8/28/2014 14:36'!
moveObject: aString To: aPoint 
	"side effect.  PRE object exists"
	| o |
	o := (self objectNamed: aString) .
	o moveTo: aPoint bearing: o bearing  zoom: zoom! !

!AgvDSLDrawing methodsFor: 'interface-agv' stamp: 'hk 8/28/2014 14:32'!
objectsMatching: aString 
	"answer a Dictionary of all objects named ..aString.. "
	
	^ objectDict associationsSelect: [ :a | ('*', aString, '*') match: a key ]
	! !

!AgvDSLDrawing methodsFor: 'interface-agv' stamp: 'hk 8/28/2014 14:32'!
objectsMatching: aString within: aRadius of: aPoint 
	"answer a Dictionary of all objects named ..aString.. within aRadius around aPoint."

	^ (objectDict
		associationsSelect: [ :a | ('*', aString, '*') match: a key ])
		select: [:v | (v location asNonFractionalPoint - aPoint) r  < aRadius]! !

!AgvDSLDrawing methodsFor: 'interface-agv' stamp: 'hk 8/28/2014 14:38'!
objectsWithin: aRadius of: aPoint 
	"answer a Dictionary of all objects within aRadius around aPoint.
	PRE the boundary is a pathshape)"
	
	^ (objectDict
		select: [:v | (v isKindOf: GeoDSLPathshape) not])
		select: [:v | (v location asFloat - aPoint) r  < aRadius]! !

Announcement subclass: #AgvOccupied
	instanceVariableNames: 'agvNo'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AGV'!

!AgvOccupied commentStamp: '<historical>' prior: 0!
An AgvOccupied  is an Announcement .

Instance Variables
	agvNo:		<String>
!

!AgvOccupied class methodsFor: 'instance creation' stamp: 'hk 8/24/2014 15:39'!
no: agvNo 
	^self new agvNo: agvNo! !

!AgvOccupied methodsFor: 'accessing' stamp: 'hk 8/24/2014 14:22'!
agvNo
	^ agvNo! !

!AgvOccupied methodsFor: 'accessing' stamp: 'hk 8/24/2014 14:22'!
agvNo: anObject
	agvNo := anObject! !

!AgvOccupied methodsFor: 'converting' stamp: 'hk 8/24/2014 14:31'!
asString
	^ 'AGV no: ', agvNo, ' is occupied'! !

Announcement subclass: #TaskFinished
	instanceVariableNames: 'agvNo taskNo'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AGV'!

!TaskFinished commentStamp: '<historical>' prior: 0!
A TaskFinished is an Announcement .

Instance Variables
	agvNo:		<String>
	taskNo:		<String>
!

!TaskFinished class methodsFor: 'instance creation' stamp: 'hk 8/24/2014 14:16'!
for: taskNo on: agvNo
	^self new 
		taskNo: taskNo; 
		agvNo: agvNo! !

!TaskFinished methodsFor: 'accessing' stamp: 'hk 8/24/2014 14:13'!
agvNo
	^ agvNo! !

!TaskFinished methodsFor: 'accessing' stamp: 'hk 8/24/2014 14:13'!
agvNo: anObject
	agvNo := anObject! !

!TaskFinished methodsFor: 'converting' stamp: 'hk 8/24/2014 14:31'!
asString
	^ 'task ', taskNo, ' for AGV ', agvNo, ' is finished'! !

!TaskFinished methodsFor: 'accessing' stamp: 'hk 8/24/2014 14:13'!
taskNo
	^ taskNo! !

!TaskFinished methodsFor: 'accessing' stamp: 'hk 8/24/2014 14:13'!
taskNo: anObject
	taskNo := anObject! !

Object subclass: #AGV
	instanceVariableNames: 'x v ambitAgvs ambitLanes ambitBorder task target name1 locked lastTarget lastDir targetDirExit targetDirEntry tracked fList'
	classVariableNames: 'AGVs Area FAGVrep FBorder FLanerep FTargetrep Ffunnel Flane Friction MyAnnouncer Radius Simulation Vmax Vmin'
	poolDictionaries: ''
	category: 'AGV'!

!AGV commentStamp: '<historical>' prior: 0!
An AGV moves according to forces from its environment.

Instance Variables 
	ambitAgvs:		<Coll of Points>	all AGVs that repel me
	ambitLanes:	<Coll of Points>	all Lanes that repel me
	ambitBorder:	<Coll of Points>	all border points that repel me
	v:				<Point>			m/s
	x:				<Point>			m 
	task:			<Task | nil>	
	target:			<Point | nil>		Cache for target coordinate
	targetDirExit 	<Point>			unit point
	targetDirEntry 	<Point>			unit point
	lastTarget		<Point | nil>		target coordinate after unload
	lastDir 			<Point | nil>		unit point
	name1:			<String>
	locked:			<Integer>			count down to 0 = unlock
							
	fList 			<OrderedCollection of Array(2) of Points> 		DSL debugging
	tracked			<Boolean>										DSL debugging
							
Forces act between the AGV and 
i) boundary, other AGVs and other lanes - repulsive 1/r^2
ii) loadLane / unloadLane - attractive constant before visit and 
						  - repulsive after visit = i) 
Friction surrogate:
see self checkTarget and self maxV

Class variables
	AGVs 		<Dictionary of: name1 and AGV>	
	Flane 		<Number>		attractive constant force to target lane
	Ffunnel 		<Number>		funnel force to target lane
	FTargetrep	<Number>		repulsive force from lastTarget
	FAGVrep	<Number>		repulsive 1/r^2 force from other agv
	FLanerep	<Number>		repulsive 1/r^2 force from other lanes
	FBorder	<Number>		repulsive 1/r^2 force from border points
	Vmax		<Number>		>0
	Vmin		<Number>		>0  -  used for target approach
	Friction		<Number>		determines maximal speed  
	Radius 		<Number>		of the environment of forces (ambit)
	Area 		interface object that ca be asked
				- for the ambit (environment)
				- for lane coordinates
				- to update own coordinates
				- to add or remove an AGV
	MyAnnouncer 		
				interface mediator
				
Todo: =========================================
- Tests 
!

!AGV class methodsFor: 'interface-geo' stamp: 'hk 8/28/2014 14:32'!
AGVsFromArea
	Area ifNil: [ ^self ]. 
	(Area objectsMatching: 'agv') keysAndValuesDo: 
		[ :k :v | AGVs add: (Association key: k value: (self named: k at: v location)) ]! !

!AGV class methodsFor: 'private' stamp: 'hk 9/5/2014 20:59'!
agvAt: agvNo 
	"AGV at: 'agv03'  "

	^(AGVs at: agvNo) 
	! !

!AGV class methodsFor: 'interface-simulation' stamp: 'hk 8/25/2014 12:37'!
allAGVs
	"answers a Dictionary of the AGV names with [location, status] as value"

	^AGVs collect: [ :v | Array with: v location  with: v free  ]! !

!AGV class methodsFor: 'class initialization' stamp: 'hk 8/24/2014 15:22'!
announcer: anAnn
	MyAnnouncer := anAnn! !

!AGV class methodsFor: 'private' stamp: 'hk 9/2/2014 16:33'!
arrowFrom: aPoint with: aVector 

^ (PolygonMorph vertices: (Array with: aPoint with: (aPoint + aVector)) color: Color black borderWidth: 1 borderColor: Color black) makeForwardArrow

! !

!AGV class methodsFor: 'private' stamp: 'hk 9/2/2014 15:45'!
centerOf: anArrayOfPointsAndVectors 
	"AGV centerOf: { 	{(300@150). (0@20)} . 
					  	{(300@150). (20@20)} .
					 	{(300@150). (50@0)} .  
						{(200@150). (50@ -50)}   } "
	| center s |
	center := 0@0.
	s := anArrayOfPointsAndVectors size.
	1 to: s do: [ :i  | center := center +  (anArrayOfPointsAndVectors at: i) first ] .
	^center / s 
! !

!AGV class methodsFor: 'visualisation' stamp: 'hk 9/7/2014 12:03'!
clearFieldFor: anAgvName

	(self agvAt: anAgvName ) clearFList 
! !

!AGV class methodsFor: 'private' stamp: 'hk 9/2/2014 16:33'!
doubleArrowFrom: aPoint with: aVector 

^(PolygonMorph vertices: (Array with: aPoint with: (aPoint + aVector)) color: Color black borderWidth: 1 borderColor: Color black) makeBothArrows

! !

!AGV class methodsFor: 'class initialization' stamp: 'hk 9/8/2014 12:55'!
initialize 
	AGVs := Dictionary new.
	Flane := 5 .
	FTargetrep := 100 .
	Ffunnel := 100.
	FAGVrep := 200 .
	FLanerep := 0 .
	FBorder := 100 .
	Friction := 100 .
	Vmax := 10 .
	Vmin := 0.5 .
	Radius := 50 .
! !

!AGV class methodsFor: 'private' stamp: 'hk 9/2/2014 17:02'!
lengthOfForce: anArrayOfPointsAndVectors 
	"AGV lengthOfForce: { 	{(300@150). (0@20)} . 
					  	{(300@150). (20@20)} .
					 	{(300@150). (50@0)} .  
						{(200@150). (50@ -50)}   }  "
	| radius |
	radius := 0.
	1 to: anArrayOfPointsAndVectors size do: [ :i  |   | p |
		p := (anArrayOfPointsAndVectors at: i) second .
		radius := (radius max: p x abs) max: p y abs ] .
	^radius 
! !

!AGV class methodsFor: 'instance creation' stamp: 'hk 7/4/2014 16:24'!
named: aName at: aPoint 
	^(self new) 
		location: aPoint;
		name1: aName;
		yourself! !

!AGV class methodsFor: 'interface-simulation' stamp: 'hk 7/3/2014 12:38'!
next
	AGVs do: [ :e |  e next ]! !

!AGV class methodsFor: 'private' stamp: 'hk 9/2/2014 15:59'!
radiusOf: anArrayOfPointsAndVectors center: aPoint
	"AGV radiusOf: { 	{(300@150). (0@20)} . 
					  	{(300@150). (20@20)} .
					 	{(300@150). (50@0)} .  
						{(200@150). (50@ -50)}   } center: (275@150)  "
	| radius s |
	radius := 0.
	s := anArrayOfPointsAndVectors size.
	1 to: s do: [ :i  |   | p |
		p := (anArrayOfPointsAndVectors at: i) first .
		radius := (radius max: (p-aPoint) x abs) max: (p-aPoint) y abs ] .
	^radius 
! !

!AGV class methodsFor: 'class initialization' stamp: 'hk 7/2/2014 15:53'!
setArea: anArea 
	Area := anArea! !

!AGV class methodsFor: 'interface-simulation' stamp: 'hk 9/6/2014 22:53'!
showFListFor: agvNo 
	"AGV showFListFor: 'agv01' "

	| agv |
	agv := (AGVs at: agvNo).
	AGV showField: agv fList for: agvNo ! !

!AGV class methodsFor: 'private' stamp: 'hk 9/6/2014 22:55'!
showField: anArrayOfPointsAndVectors for: anAgvName
	"AGV showField: { 	{(300@150). (0@20)} . 
					  	{(300@150). (20@20)} .
					 	{(300@150). (50@0)} .  
						{(200@150). (50@ -50)}   } "
	|win canvas newScale multiple center radius scale lof fScale newfScale fmultiple|
	anArrayOfPointsAndVectors ifEmpty: [ ^GrowlMorph 
		openWithLabel: 'no data tracked' translated contents: '' ] .
	center := AGV centerOf: anArrayOfPointsAndVectors .
	radius := AGV radiusOf: anArrayOfPointsAndVectors center: center.
	(scale := 100 / radius) > 100 
		ifTrue: 	[ multiple := (scale/100) truncated. newScale := scale/multiple ] 
		ifFalse: [ multiple := (100/scale) truncated. newScale := scale*multiple ].
	lof := AGV lengthOfForce: anArrayOfPointsAndVectors .
	(fScale := 100 / lof) > 100 
		ifTrue: 	[ fmultiple := (fScale/100) truncated. newfScale := fScale/fmultiple ] 
		ifFalse: [ fmultiple := (100/fScale) truncated. newfScale := fScale*fmultiple ].
	win:= StandardWindow labelled: 'Forces'.
	canvas := Morph new.    
	1 to: anArrayOfPointsAndVectors size do: [ :i  |  | puv |
	puv := (anArrayOfPointsAndVectors at: i) .
	canvas addMorph:  
		(self arrowFrom: (puv first - center) *scale + (250@250) with: puv second * fScale )  ] .
	canvas addMorph: (self doubleArrowFrom: (5@5) with: (newScale@0)).
	canvas addMorph: ( anAgvName asMorph position: (240@10)).
	canvas addMorph: (( 'Scale: ', multiple asString) asMorph position: (10@10)).
	canvas addMorph: ( center x  truncated asMorph position: (245@450)).
	canvas addMorph: ( center y  truncated asMorph position: (5@230)).
	canvas addMorph: (self doubleArrowFrom: ((480 - newfScale) @5) with: (newfScale@0)).
	canvas addMorph: (( 'Force: ', fmultiple asString) asMorph position: (420@10)).
	win addMorph: canvas frame: (0@0 corner: 1@1). 
	win openInWorld. win extent: (500@500). 
! !

!AGV class methodsFor: 'visualisation' stamp: 'hk 9/6/2014 22:52'!
showFieldFor: anAgvName
	"AGV showFieldFor: 'agv02' "

	self showField: (self agvAt: anAgvName ) fList for: anAgvName
! !

!AGV class methodsFor: 'visualisation' stamp: 'hk 9/7/2014 11:49'!
showFunnelForceAtDistance: dist
	"AGV showFunnelForceAtDistance: 20 "

	|uV arr agv |
	uV := 0@1.		"to south"
	agv := self new.
	arr := OrderedCollection new.
	1 to: 36 do: [ :i |  | rV  f |
		rV := Point r: dist degrees:  (10 * i) .
		f := agv forceFunnel: rV negated direction: uV.   "toGoal"
		arr add: (Array with: rV with: f) ].
	self showField: arr for: 'nil'
! !

!AGV class methodsFor: 'interface-simulation' stamp: 'hk 8/24/2014 15:41'!
task: agvNo from: lane1 to: lane2 
	"AGV task: 'agv03' from: 'laneCB1S1'  to: 'laneLB2S3' "

	| agv |
	agv := (AGVs at: agvNo).
	agv free ifFalse: [ MyAnnouncer announce: (AgvOccupied no: agvNo)].
	agv
		newTask: lane1, lane2
		from: lane1
		to: lane2
	! !

!AGV class methodsFor: 'interface-simulation' stamp: 'hk 9/5/2014 20:56'!
trackAgv: agvNo 
	"AGV at: 'agv03'  "

	^(AGVs at: agvNo) track
	! !

!AGV class methodsFor: 'interface-simulation' stamp: 'hk 9/5/2014 20:56'!
untrackAgv: agvNo 
	"AGV at: 'agv03'  "

	^(AGVs at: agvNo) untrack
	! !

!AGV methodsFor: 'simulation' stamp: 'hk 9/4/2014 14:58'!
adjustToTarget:  aV
	"the magnitude of aV shall not exceed the velocity needed 
	to reach the lane in the next step; 0.5 < a < 1.
	answer a modified vector"

	^((target notNil) and: [aV r > (target - x -aV) r ])
		ifTrue: [0.7 * (target-x)] ifFalse: [ aV ]
! !

!AGV methodsFor: 'simulation core' stamp: 'hk 9/5/2014 16:52'!
checkTarget
	"loadLane / unloadLane will be reached when v=dx < vmin.
	adjust x, load or unload, switch task"
	"PRE target, task isNotNil"

	(target-x) r > Vmin ifTrue: [ ^self ].			"not yet there"
	self updateX .
	v := 0@0. 
	self lock
	! !

!AGV methodsFor: 'accessing' stamp: 'hk 9/7/2014 12:01'!
clearFList
	fList := OrderedCollection new! !

!AGV methodsFor: 'private' stamp: 'hk 8/29/2014 15:40'!
clearLastTarget
		lastTarget := lastDir := nil.
	! !

!AGV methodsFor: 'private' stamp: 'hk 8/29/2014 17:49'!
copyTarget
		lastTarget := target.
		lastDir := targetDirExit.
	! !

!AGV methodsFor: 'testing' stamp: 'hk 6/30/2014 11:24'!
empty
	^ task empty! !

!AGV methodsFor: 'accessing' stamp: 'hk 9/4/2014 22:21'!
fList
	^fList! !

!AGV methodsFor: 'testing' stamp: 'hk 7/2/2014 12:13'!
finished
	^ task finished! !

!AGV methodsFor: 'simulation support' stamp: 'hk 8/28/2014 14:32'!
flushAmbit
	| agvDict laneDict |
	agvDict := (Area objectsMatching: 'agv' within: Radius of: x).
	agvDict removeKey: name1.								"must be there"
	ambitAgvs := agvDict values collect: [:o | o location].

	laneDict := (Area objectsMatching: 'lane' within: Radius of:  x) .
	laneDict removeKey: self targetLane ifAbsent: [  ].			"maybe 'none' "
	ambitLanes := laneDict values collect: [:o | o location].
	
	ambitBorder := Area boundaryPointsWithin: Radius of:  x.
	 ! !

!AGV methodsFor: 'simulation support' stamp: 'hk 9/8/2014 15:00'!
flushTarget
	"to be called after task progress"
 
	| tgt dir |
	tgt := Area objectNamed: self targetLane.
	target := tgt location.
	dir := (tgt bearing cos) @ (tgt bearing sin).
	targetDirExit := dir.
	targetDirEntry := ('*','LB','*' match: self targetLane ) 
						ifTrue: [ dir negated ]
						ifFalse: [ dir ]
	 
! !

!AGV methodsFor: 'simulation' stamp: 'hk 9/8/2014 12:54'!
forceAgvFrom: aPoint
	"the repulsive force is directed from aPoint to x"
	"after load or unload the agv may be at the target,
	hence we avoid to divide by Zero"
	
	| d |
	d := x squaredDistanceTo:  aPoint.
	^d > 1 
		ifTrue: [ ^FAGVrep asFloat * (x - aPoint) /  d ]
		ifFalse: [ ^FAGVrep asFloat * (x - aPoint) ]
	 ! !

!AGV methodsFor: 'simulation' stamp: 'hk 9/8/2014 12:55'!
forceBorderFrom: aPoint
	"the repulsive force is directed from aPoint to x"
	"after load or unload the agv may be at the target,
	hence we avoid to divide by Zero"
	
	| d |
	d := x squaredDistanceTo:  aPoint.
	^d > 1 
		ifTrue: [ ^FBorder asFloat * (x - aPoint) /  d ]
		ifFalse: [ ^FBorder asFloat * (x - aPoint) ]
	 ! !

!AGV methodsFor: 'simulation' stamp: 'hk 9/8/2014 12:53'!
forceFrom: aPoint direction: aUnitPoint
	"the repulsive force is aligned with aUnitPoint"
	
	| d |
	(lastTarget dist: x) > Radius ifTrue: [ lastTarget := nil. ].		"we are off !! "
	d := aPoint dist: x. 
	^FTargetrep asFloat / d * aUnitPoint .! !

!AGV methodsFor: 'simulation' stamp: 'hk 9/7/2014 11:45'!
forceFunnel: toGoal direction: aUnitVector
	"the guiding funnel is aligned with aUnitPoint"
	
	| d  s dirDiff  fv |  
	d := toGoal r .
	dirDiff := (toGoal theta - aUnitVector theta) abs.
	s := 2 min: dirDiff.											"the funnel strenght "
	fv :=  toGoal normal.										"go around clockwise"
	(toGoal sideOf: aUnitVector) = #left ifTrue: [ fv :=  fv negated].	"or counterclockwise"
	^Ffunnel * fv * s / d  ! !

!AGV methodsFor: 'simulation' stamp: 'hk 9/8/2014 12:55'!
forceLanesFrom: aPoint
	"the repulsive force is directed from aPoint to x"
	"after load or unload the agv may be at the target,
	hence we avoid to divide by Zero"
	
	| d |
	d := x squaredDistanceTo:  aPoint.
	^d > 1 
		ifTrue: [ ^FLanerep asFloat * (x - aPoint) /  d ]
		ifFalse: [ ^FLanerep asFloat * (x - aPoint) ]
	 ! !

!AGV methodsFor: 'simulation' stamp: 'hk 9/4/2014 14:27'!
forceTo: aPoint
	"the attractive force is directed to aPoint, 
	modified by the funnel around the goal"
	
	^ Flane asFloat * (aPoint -x) normalized  
	+ (self forceFunnel:  (target - x) direction: targetDirEntry)! !

!AGV methodsFor: 'testing' stamp: 'hk 7/2/2014 12:09'!
free
	^task = nil! !

!AGV methodsFor: 'simulation' stamp: 'hk 9/4/2014 22:08'!
friction
	^1 -  ( 1 min: (v dotProduct: v) /Friction )! !

!AGV methodsFor: 'initialize-release' stamp: 'hk 9/7/2014 12:02'!
initialize 	
	v := 0@0.
	x := 0@0.
	locked := 0.
	ambitAgvs  := #().
	ambitLanes  := #().
	ambitBorder := #().
	self clearFList .
	tracked := false
	! !

!AGV methodsFor: 'testing' stamp: 'hk 6/30/2014 11:24'!
loaded
	^ task loaded! !

!AGV methodsFor: 'accessing' stamp: 'hk 7/2/2014 16:10'!
location
	^x! !

!AGV methodsFor: 'accessing' stamp: 'hk 7/4/2014 16:24'!
location: aPoint 
	x := aPoint! !

!AGV methodsFor: 'simulation core' stamp: 'hk 7/8/2014 12:48'!
lock
	locked := 5
	! !

!AGV methodsFor: 'testing' stamp: 'hk 7/8/2014 12:49'!
locked
	^locked > 0
	! !

!AGV methodsFor: 'simulation' stamp: 'hk 8/25/2014 16:53'!
maxV:  aV
	"the magnidute of aV does not exceed vmax.
	answer a modified vector"

	^(aV r < Vmax) ifTrue: [ aV ] ifFalse: [ aV * Vmax / (aV r) ] ! !

!AGV methodsFor: 'accessing' stamp: 'hk 7/2/2014 13:51'!
name1: aName 
	name1 := aName! !

!AGV methodsFor: 'simulation' stamp: 'hk 8/28/2014 15:28'!
newTask: partialID from: a to: b
	"assign a new task"
	
	task := Task id: partialID from: a to: b.
	self clearLastTarget; flushTarget; updateEmpty 
! !

!AGV methodsFor: 'simulation' stamp: 'hk 9/5/2014 16:53'!
next
	
	self readyForSwitch ifTrue: [ ^self switchTask ].
	self locked ifTrue: [ ^self wait ].  
	v := v * self friction.
	v := v + (self sumOfForces).			"f ~ dv; m=dt =1"
	v := (self maxV: v) .						"not faster"
	self setV: (self adjustToTarget: v).			"at target slow down appropriately"
	x := (x + v) asFloatPoint.		" move "
	self updateX.
	task ifNotNil: [self checkTarget]		" target reached? "
	! !

!AGV methodsFor: 'testing' stamp: 'hk 7/2/2014 12:12'!
occupied
	^self free not! !

!AGV methodsFor: 'testing' stamp: 'hk 7/8/2014 12:51'!
readyForSwitch
	^locked = 1
	! !

!AGV methodsFor: 'interface-simulation' stamp: 'hk 8/25/2014 12:43'!
reportTask: id
	MyAnnouncer announce: (TaskFinished for:  id on: name1).! !

!AGV methodsFor: 'accessing' stamp: 'hk 9/5/2014 17:49'!
setV: aVelocityPoint
	v := aVelocityPoint .
	tracked ifTrue: [ fList addLast: (Array with: x with: v) ]! !

!AGV methodsFor: 'simulation' stamp: 'hk 9/6/2014 17:37'!
sumOfForces
	| f |
	f := 0@0.
	self flushAmbit.
	ambitAgvs do: [ :p | f := f + (self forceAgvFrom: p) ].
	ambitLanes do: [ :p | f := f + (self forceLanesFrom: p) ].		
	ambitBorder do: [ :p | f := f + (self forceBorderFrom: p) ].
	target ifNotNil: [ f := f + (self forceTo:target) ].
	lastTarget ifNotNil: [ f := f + (self forceFrom: lastTarget direction: lastDir) ].
	^f asFloatPoint ! !

!AGV methodsFor: 'simulation' stamp: 'hk 8/28/2014 15:26'!
switchTask
	"load or unload, switch task"
	"PRE Task isNotNil"

	| id |
	self unlock.
	task next.									"do the target action"
	self loaded ifTrue: 
		[ self copyTarget; flushTarget; updateLoaded ].		"unload target"
	self finished ifTrue: 
		[id := task id.
		self copyTarget .
		target := task := nil .					"update  AGV data, then  report"
		self reportTask: id; updateFree ].
	! !

!AGV methodsFor: 'accessing' stamp: 'hk 7/2/2014 12:03'!
targetLane
	self free	ifTrue: [ ^'none' ].
	self empty 	ifTrue: [ ^task loadLane].
	self loaded ifTrue: [ ^task unloadLane  ]! !

!AGV methodsFor: 'interface-simulation' stamp: 'hk 9/5/2014 17:03'!
track
	tracked := true! !

!AGV methodsFor: 'simulation' stamp: 'hk 7/8/2014 12:54'!
unlock
	locked := 0
	! !

!AGV methodsFor: 'interface-simulation' stamp: 'hk 9/5/2014 17:03'!
untrack
	tracked := false! !

!AGV methodsFor: 'interface-geo' stamp: 'hk 7/7/2014 15:06'!
updateEmpty
	Area ifNotNil: [ Area change: name1 at: #color: to: Color yellow]! !

!AGV methodsFor: 'interface-geo' stamp: 'hk 7/8/2014 17:19'!
updateFree
	Area ifNotNil: [ Area change: name1 at: #color: to: Color lightGreen]! !

!AGV methodsFor: 'interface-geo' stamp: 'hk 7/2/2014 13:41'!
updateLoaded
	Area ifNotNil: [ Area change: name1 at: #color: to: Color red]! !

!AGV methodsFor: 'interface-geo' stamp: 'hk 7/2/2014 13:28'!
updateX
	Area ifNotNil: [ Area  moveObject: name1 To: x ]! !

!AGV methodsFor: 'simulation' stamp: 'hk 7/8/2014 12:48'!
wait
	locked := locked - 1
	! !

Object subclass: #AgvConfigurator
	instanceVariableNames: 'dispatcher visualisation agvUI geoWindow'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AGV-SIM'!

!AgvConfigurator commentStamp: '<historical>' prior: 0!
dock addMorphBack: self simulationToolbar .
	dock addMorphBack: self agvToolbar .
An AgvConfigurator is the component that configures the agv (simulation) system: init and start-up.

Instance Variables
	dispatcher:		<AgvDispatcher>
	visualisation:	<GeoDSLDrawing>
	geoWindow:	<StandardWindow>  shows the Terminal Layout
	agv:			global class, no slot needed

Todo: =========================================
- Tests 
- UI mit start/stop  und Aufträgen (Tasks)!

!AgvConfigurator class methodsFor: 'example' stamp: 'hk 8/23/2014 13:59'!
run
	"AgvConfigurator run"
	AgvConfigurator new! !

!AgvConfigurator methodsFor: 'initialize-release' stamp: 'hk 8/25/2014 18:08'!
initialize 
	"AgvConfigurator new "
	
	| geoModel ann |
	AGV initialize; announcer: (ann := Announcer new).
	Task initialize.
	
	GeoDSLCompiler drawingClass:  (Smalltalk at: #AgvDSLDrawing) .
	geoModel := GeoDSLModel new.	 
	visualisation := geoModel drawingFrom: self simpleTerminalAGVArea.
	AGV setArea: visualisation; AGVsFromArea.	
	(dispatcher := AgvDispatcher new) getAgvs .
	AgvUI announcer: ann.
	agvUI := AgvUI on: self and: dispatcher. 

	geoWindow := geoModel open.
	agvUI open.
! !

!AgvConfigurator methodsFor: 'initialize-release' stamp: 'hk 8/18/2014 16:03'!
release 
geoWindow close.
dispatcher release! !

!AgvConfigurator methodsFor: 'geoSpec' stamp: 'hk 8/19/2014 17:47'!
simpleTerminalAGVArea
	^ '      			
	displayIn: TerminalArea size: 1000@500    	"defines the initial zoom"
	"AGVs"
		(ellipse:  agv01 size: 20@20 at: 100@150 label: agv1 color: lightGreen)
		(ellipse:  agv02 size: 20@20 at: 140@150 label: agv2 color: lightGreen)
		(ellipse:  agv03 size: 20@20 at: 160@150 label: agv3 color: lightGreen)
		(ellipse:  agv04 size: 20@20 at: 180@150 label: agv4 color: lightGreen)
		(ellipse:  agv05 size: 20@20 at: 200@150 label: agv5 color: lightGreen)
		(ellipse:  agv06 size: 20@20 at: 250@150 label: agv6 color: lightGreen)
		(ellipse:  agv07 size: 20@20 at: 280@150 label: agv7 color: lightGreen)
		(ellipse:  agv08 size: 20@20 at: 300@150 label: agv8 color: lightGreen)
		(ellipse:  agv09 size: 20@20 at: 320@150 label: agv9 color: lightGreen)
	"Lagerblock-Spuren"
		(rectangle:  laneLB1S1 size: 60@20 at: 360@60  bearing: 90   label: spur1 color: lightBlue)
		(rectangle:  laneLB1S2 size: 60@20 at: 380@60  bearing: 90   label: spur2 color: lightBlue)
		(rectangle:  laneLB1S3 size: 60@20 at: 400@60  bearing: 90   label: spur3 color: lightBlue)
		(rectangle:  laneLB1S4 size: 60@20 at: 420@60  bearing: 90   label: spur4 color: lightBlue)
		(rectangle:  laneLB2S1 size: 60@20 at: 760@60  bearing: 90   label: spur1 color: lightBlue)
		(rectangle:  laneLB2S2 size: 60@20 at: 780@60  bearing: 90   label: spur2 color: lightBlue)
		(rectangle:  laneLB2S3 size: 60@20 at: 800@60  bearing: 90   label: spur3 color: lightBlue)
		(rectangle:  laneLB2S4 size: 60@20 at: 820@60  bearing: 90   label: spur4 color: lightBlue)
	"CB-Spuren"	
		(rectangle:  laneCB1S1 size: 60@20 at: 360@240  label: spur1 color: lightGreen)
		(rectangle:  laneCB1S2 size: 60@20 at: 360@260  label: spur2 color: lightGreen)
		(rectangle:  laneCB2S1 size: 60@20 at: 460@240  label: spur1 color: lightGreen)
		(rectangle:  laneCB2S2 size: 60@20 at: 460@260  label: spur2 color: lightGreen)
		(rectangle:  laneCB3S1 size: 60@20 at: 760@240  label: spur1 color: lightGreen)
		(rectangle:  laneCB3S2 size: 60@20 at: 760@260  label: spur2 color: lightGreen)
		(rectangle:  laneCB4S1 size: 60@20 at: 860@240  label: spur1 color: lightGreen)
		(rectangle:  laneCB4S2 size: 60@20 at: 860@260  label: spur2 color: lightGreen)
	"Boundary"	
		(polygon: boundary [ 0@0  30@0  60@0  90@0  120@0  150@0  180@0  210@0  240@0  270@0  300@0 330@0  360@0  390@0  420@0  450@0  480@0  510@0  540@0  570@0  600@0  630@0  660@0  690@0  720@0  750@0  780@0  810@0  840@0  870@0  900@0  930@0  960@0  990@0 990@30  990@60  990@90  990@120  990@150  990@180  990@210  990@240  990@270   990@300  960@300  930@300  900@300  870@300  840@300  810@300  780@300  750@300  720@300  690@300  660@300  630@300  600@300  570@300  540@300  510@300  480@300  450@300  420@300  390@300  360@300  330@300  300@300  270@300  240@300  210@300  180@300  150@300  120@300  90@300  60@300  30@300  0@300 0@270   0@240   0@210   0@180   0@150   0@120   0@90   0@60   0@30   ] )
'! !

!AgvConfigurator methodsFor: 'accessing' stamp: 'hk 8/22/2014 18:54'!
visualisation
	^ visualisation! !

Object subclass: #AgvDispatcher
	instanceVariableNames: 'freeAgvs occAgvs freeTasks simulator running'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AGV-SIM'!

!AgvDispatcher commentStamp: '<historical>' prior: 0!
An AgvDispatcher dispatches transport tasks to agvs.
It allows to:  
- issue a new transport task
- start or stop the simulation
the AGV component ist (as a class) globally known.
the location of the agvs will be used by sophisticated optimizers.

Instance Variables
	freeAgvs:		<Dict of: Agv-Name and location>		free
	occAgvs:		<Dict of: Agv-Name and location>		occupied
	freeTasks:		<Dict of: Task>
	simulator:		<Block>
	running: 		<Boolean>				tested by the simulator block
							

Todo: =========================================
- Tests !

!AgvDispatcher methodsFor: 'accessing' stamp: 'hk 9/5/2014 17:56'!
allAgvs
	^ AGV allAGVs collect: [ :v | v at: 1 ].
 ! !

!AgvDispatcher methodsFor: 'accessing' stamp: 'hk 8/22/2014 18:02'!
freeAgvs
	^ (AGV allAGVs select: [ :v | v at: 2 ]) collect: [ :v | v at: 1 ].
 ! !

!AgvDispatcher methodsFor: 'interface-agv' stamp: 'hk 8/22/2014 18:00'!
getAgvs
	freeAgvs := self freeAgvs .
	occAgvs := self occAgvs 
	! !

!AgvDispatcher methodsFor: 'initialize-release' stamp: 'hk 7/4/2014 14:11'!
initialize 
	self stop.
	simulator := [ [true] whileTrue: [self step]] forkAt: Processor activePriority+1 .
	freeTasks := Dictionary new.
	occAgvs := Dictionary new.
	freeAgvs := Dictionary new.
	! !

!AgvDispatcher methodsFor: 'interface-agv' stamp: 'hk 8/22/2014 18:02'!
occAgvs
	^(AGV allAGVs select: [ :v | (v at: 2) not ]) collect: [ :v | v at: 1 ]
	! !

!AgvDispatcher methodsFor: 'initialize-release' stamp: 'hk 8/24/2014 15:44'!
release 
	AGV setArea: nil.
	simulator terminate. 
	simulator := nil.
! !

!AgvDispatcher methodsFor: 'simulation' stamp: 'hk 7/2/2014 20:36'!
start 
	running := true ! !

!AgvDispatcher methodsFor: 'simulation' stamp: 'hk 8/21/2014 13:51'!
step 
	running ifTrue: 	[AGV next].
	(Delay forMilliseconds: 400) wait ! !

!AgvDispatcher methodsFor: 'simulation' stamp: 'hk 7/2/2014 20:36'!
stop 
	running := false ! !

Object subclass: #AgvUI
	instanceVariableNames: 'configurator dispatcher chosenAgv pickedAgv chosenLane1 chosenLane2 lanes agvDropList agvDropList2 freeAgvs allAgvs window extent statusMorph'
	classVariableNames: 'AgvAnnouncer'
	poolDictionaries: ''
	category: 'AGV-SIM'!

!AgvUI commentStamp: '<historical>' prior: 0!
An AgvUI is the UI for the AGV simulation.

Instance Variables
	window:			<StandardWindow>	the GUI window
	extent:				<Point>				window extent
	statusMorph:		<TextMorph>			status line
	agvDropList:		<DropListMorph>		need this to be able to reset
	freeAgvs:			<Array of String>		for agv chooser
	chosenAgv:			<Integer>				for agv chooser
	lanes:				<Array of String>		for lane chooser
	chosenLane1:		<Integer>				for lane chooser (pick up)
	chosenLane2:		<Integer>				for lane chooser (destination))
	configurator:		<AgvConfigurator>		
	dispatcher:			<AgvDispatcher>		

Class variables
	AgvAnnouncer 		<Announcer>			for registered events!

!AgvUI class methodsFor: 'instance creation' stamp: 'hk 8/25/2014 12:12'!
announcer: anAnn 
	"AgvAnnouncer open"
	
	AgvAnnouncer := anAnn! !

!AgvUI class methodsFor: 'instance creation' stamp: 'hk 8/23/2014 14:33'!
on: anAgvConfigurator and: aAgvDispatcher
	^self new initializeWith: anAgvConfigurator with: aAgvDispatcher
! !

!AgvUI methodsFor: 'ui' stamp: 'hk 8/25/2014 11:47'!
agvToolbar

	   ^window newToolbar: 
		{window
			newButtonFor: self
			getState: nil
			action: #task:from:to:
			arguments: #('agv06' 'laneCB2S2' 'laneLB2S3')
			getEnabled: nil
			label: 'agv06:22-23'
			help: 'with agv06 from CB2S2 to LB2S3'. 
		window
			newButtonFor: self
			getState: nil
			action: #task:from:to:
			arguments: #('agv04' 'laneCB1S1' 'laneLB1S1')
			getEnabled: nil
			label: 'agv04:11-11'
			help: 'with agv04 from CB1S1 to LB1S1'. 
		window
			newButtonFor: self
			getState: nil
			action: #task:from:to:
			arguments: #('agv05' 'laneCB2S2' 'laneLB2S3')
			getEnabled: nil
			label: 'agv05:22-23'
			help: 'with agv05 from CB2S2 to LB2S3'. 
		window
			newButtonFor: self
			getState: nil
			action: #task:from:to:
			arguments: #('agv03' 'laneCB1S1' 'laneLB1S3')
			getEnabled: nil
			label: 'agv03:11-13'
			help: 'with agv03 from CB1S1 to LB1S3'. 
 }
! !

!AgvUI methodsFor: 'ui' stamp: 'hk 9/5/2014 17:58'!
allAgvs
	^ allAgvs ifNil: [allAgvs := dispatcher allAgvs keys asOrderedCollection asSortedCollection]! !

!AgvUI methodsFor: 'ui' stamp: 'hk 8/22/2014 18:36'!
chosenAgv
	^chosenAgv ifNil: [0]! !

!AgvUI methodsFor: 'ui' stamp: 'hk 8/22/2014 18:36'!
chosenAgv: anIndex
	chosenAgv  := anIndex .
	self changed: #inputOK! !

!AgvUI methodsFor: 'ui' stamp: 'hk 8/22/2014 18:36'!
chosenLane1
	^chosenLane1 ifNil: [0]! !

!AgvUI methodsFor: 'ui' stamp: 'hk 8/22/2014 18:36'!
chosenLane1: anIndex
	chosenLane1  := anIndex.
	self changed: #inputOK! !

!AgvUI methodsFor: 'ui' stamp: 'hk 8/22/2014 18:36'!
chosenLane2
	^chosenLane2 ifNil: [0]! !

!AgvUI methodsFor: 'ui' stamp: 'hk 8/22/2014 18:36'!
chosenLane2: anIndex
	chosenLane2  := anIndex .
	self changed: #inputOK! !

!AgvUI methodsFor: 'ui' stamp: 'hk 8/24/2014 20:46'!
clearAgvDropList
	freeAgvs := nil.
	self chosenAgv: 0.	
	agvDropList updateListSelectionIndex.
	agvDropList updateList 
! !

!AgvUI methodsFor: 'ui' stamp: 'hk 9/7/2014 12:05'!
clearButton
	^ window
		newButtonFor: self action: #clearField getEnabled: #input2OK label: #'Clear force field' help: 'cleanup my force data'! !

!AgvUI methodsFor: 'ui-actions' stamp: 'hk 9/7/2014 12:04'!
clearField 
	(AGV clearFieldFor:  (allAgvs at: pickedAgv) ) ! !

!AgvUI methodsFor: 'ui' stamp: 'hk 8/24/2014 20:56'!
freeAgvs
	^ freeAgvs ifNil: [freeAgvs := dispatcher freeAgvs keys asOrderedCollection asSortedCollection]! !

!AgvUI methodsFor: 'initialize-release' stamp: 'hk 9/5/2014 18:14'!
initializeWindow
	| simTool agvTool  y1 y2 tabGroup |
	window := (StandardWindow labelled: 'AGV Simulation')
				model: configurator.
	y1 := (simTool := self simulationToolbar) minExtent y.
	y2 := y1 + (agvTool := self agvToolbar) minExtent y.
	tabGroup :=  window newTabGroup: { 'New AGV Task:' -> (self taskEntry) .
											'Track Velocity'  -> (self trackingDialog )  }.
	window
		addMorph: simTool
		fullFrame: ((0 @ 0 corner: 1 @ 0) asLayoutFrame bottomOffset: y1).
	window addMorph: agvTool fullFrame: ((0 @ 0 corner: 1 @ 0) asLayoutFrame topOffset: y1;
			 bottomOffset: y2).
	window
		addMorph: tabGroup
		fullFrame: ((0 @ 0 corner: 1 @ 0) asLayoutFrame topOffset: y2).
	window
		addMorph: (statusMorph := window newText: ' ')
		fullFrame: (LayoutFrame
				fractions: (0 @ 1 corner: 1 @ 1)
				offsets: (0 @ 50 negated corner: 0 @ 0)).
	window themeChanged.
	extent := window extent + (65 @ 90)! !

!AgvUI methodsFor: 'initialize-release' stamp: 'hk 8/24/2014 20:46'!
initializeWith: aConfigurator with: aDispatcher 
	configurator := aConfigurator .
	dispatcher := aDispatcher . 
	AgvAnnouncer on: TaskFinished do: [:ann | self clearAgvDropList; newStatus: ann asString ].
	AgvAnnouncer on: AgvOccupied do: [:ann | self newStatus: ann asString ].
	self initializeWindow.
	^self




! !

!AgvUI methodsFor: 'ui' stamp: 'hk 9/5/2014 18:23'!
input2OK
	(self pickedAgv = 0) ifTrue: [ ^false ].
	^true! !

!AgvUI methodsFor: 'ui' stamp: 'hk 8/22/2014 18:36'!
inputOK
	(self chosenAgv = 0) ifTrue: [ ^false ].
	(self chosenLane1 =  0) ifTrue: [ ^false ].
	(self chosenLane2 = 0) ifTrue: [ ^false ].
	(chosenLane1 = chosenLane2 ) ifTrue: [ ^false ].
	^true! !

!AgvUI methodsFor: 'ui' stamp: 'hk 8/28/2014 14:32'!
lanes
	^ lanes ifNil: [lanes := (configurator  visualisation objectsMatching: 'lane') keys asSortedCollection ]! !

!AgvUI methodsFor: 'status' stamp: 'hk 8/23/2014 15:53'!
newStatus: aStringOrText 
 	statusMorph contents: aStringOrText


! !

!AgvUI methodsFor: 'ui' stamp: 'hk 9/5/2014 18:25'!
offButton
	^ window
		newButtonFor: self action: #trackingOff getEnabled: #input2OK label: #'Do not Track' help: 'switch tracking off for this AGV'! !

!AgvUI methodsFor: 'ui' stamp: 'hk 8/25/2014 13:13'!
okButton
	^ window
		newButtonFor: self action: #submitTask getEnabled: #inputOK label: #Submit help: 'lanes must be different'! !

!AgvUI methodsFor: 'ui' stamp: 'hk 9/5/2014 18:24'!
onButton
	^ window
		newButtonFor: self action: #trackingOn getEnabled: #input2OK label: #Track help: 'switch tracking on for this AGV'! !

!AgvUI methodsFor: 'ui' stamp: 'hk 9/6/2014 22:31'!
onOff
	^window newRow: (Array with: self onButton  with: self offButton  )! !

!AgvUI methodsFor: 'ui-actions' stamp: 'hk 8/23/2014 15:52'!
open 
	window openInWorld.
	window extent: extent


! !

!AgvUI methodsFor: 'ui' stamp: 'hk 9/5/2014 18:21'!
pickedAgv
	^ pickedAgv ifNil: [0]! !

!AgvUI methodsFor: 'ui' stamp: 'hk 9/5/2014 18:36'!
pickedAgv: anIndex
	pickedAgv := anIndex.
	self changed: #input2OK! !

!AgvUI methodsFor: 'initialize-release' stamp: 'hk 8/22/2014 20:16'!
release 
	dispatcher := nil.
	configurator release.
	configurator := nil ! !

!AgvUI methodsFor: 'ui' stamp: 'hk 9/6/2014 22:29'!
showButton
	^ window
		newButtonFor: self action: #showField getEnabled: #input2OK label: #'Show force field' help: 'display a window showing the tracked forces'! !

!AgvUI methodsFor: 'ui' stamp: 'hk 9/7/2014 12:06'!
showClear
	^window newRow: (Array with: self showButton  with: self clearButton  )! !

!AgvUI methodsFor: 'ui-actions' stamp: 'hk 9/6/2014 22:34'!
showField 
	(AGV showFieldFor:  (allAgvs at: pickedAgv) ) ! !

!AgvUI methodsFor: 'ui' stamp: 'hk 8/22/2014 19:56'!
simulationToolbar

	   ^window newToolbar: 
		{window
			newButtonFor: dispatcher
			getState: nil
			action: #start
			arguments: nil
			getEnabled: nil
			label: 'run'
			help: 'start simulation'. 
		window
			newButtonFor: dispatcher
			getState: nil
			action: #stop
			arguments: nil
			getEnabled: nil
			label: 'stop'
			help: 'stops simulation'.
		window
			newButtonFor: configurator
			getState: nil
			action: #inspect
			arguments: nil
			getEnabled: nil
			label: 'inspect'
			help: 'open inspector'.  }
! !

!AgvUI methodsFor: 'ui-actions' stamp: 'hk 8/25/2014 12:21'!
submitTask
	self 	
		task: (freeAgvs at: chosenAgv )
		from:  (lanes  at: chosenLane1 )
		to: (lanes at: chosenLane2 )
! !

!AgvUI methodsFor: 'ui-actions' stamp: 'hk 8/25/2014 11:50'!
task: agvNo from: lane1 to: lane2 
	AGV task: agvNo from: lane1  to: lane2.
	self clearAgvDropList ! !

!AgvUI methodsFor: 'ui' stamp: 'hk 8/23/2014 14:51'!
taskChoosers
	|   s d b |
	agvDropList :=  window
		newDropListFor: self
		list: #freeAgvs
		getSelected: #chosenAgv
		setSelected: #chosenAgv:
		help: 'choose on of the AGVs' .
	s :=  window
		newDropListFor: self
		list: #lanes
		getSelected: #chosenLane1
		setSelected: #chosenLane1:
		help: 'choose the pick up lane' .
	d :=  window
		newDropListFor: self
		list: #lanes
		getSelected: #chosenLane2
		setSelected: #chosenLane2:
		help: 'choose the delivery lane' .
	b := self okButton.
	^window newColumn: (Array with: agvDropList with: s with: d with: b)! !

!AgvUI methodsFor: 'ui' stamp: 'hk 9/5/2014 17:50'!
taskEntry
	^window newRow: (Array with: self taskLabels with: self taskChoosers )! !

!AgvUI methodsFor: 'ui' stamp: 'hk 8/23/2014 14:52'!
taskLabels
	|  a s d |
	
	a := window newLabel: 'AGV'.
	s := window newLabel: 'Pick Up'.
	d := window newLabel: 'destination'.
	a height: 26. s height: 26. d height: 26.
	^window newColumn: (Array with: a with: s with: d)! !

!AgvUI methodsFor: 'ui' stamp: 'hk 9/7/2014 12:07'!
trackingDialog
	agvDropList2  :=  window
		newDropListFor: self
		list: #allAgvs
		getSelected: #pickedAgv
		setSelected: #pickedAgv:
		help: 'choose one of the AGVs' .
	^window newColumn: (Array with: agvDropList2 with: self onOff with: self showClear )! !

!AgvUI methodsFor: 'ui-actions' stamp: 'hk 9/5/2014 20:56'!
trackingOff 
	(AGV untrackAgv: (allAgvs at: pickedAgv) ) ! !

!AgvUI methodsFor: 'ui-actions' stamp: 'hk 9/5/2014 20:57'!
trackingOn 
	(AGV trackAgv: (allAgvs at: pickedAgv) ) ! !

Object subclass: #Task
	instanceVariableNames: 'loadLane unloadLane stage id'
	classVariableNames: 'ID'
	poolDictionaries: ''
	category: 'AGV'!

!Task commentStamp: '<historical>' prior: 0!
A Task tells an AGV where to pick up and where to drop.
initialized with #empty

Instance Variables
	loadLane:		<Symbol>				pick up
	stage:			<#empty, #loaded, #finished>
	unloadLane:	<Symbol>				drop
	id: 				<String>				ref

Class Variables 	
	ID:				<Integer>				unique part of task id
					
Todo: =========================================
- Tests !

!Task class methodsFor: 'instance creation' stamp: 'hk 8/20/2014 18:09'!
id: partialID from: laneL to: laneU

	| t |
	t := self new.
	t id: partialID, '-', self nextID.	"make unique id"
	t loadLane: laneL.
	t unloadLane: laneU.
	^t
	! !

!Task class methodsFor: 'initialization' stamp: 'hk 8/20/2014 18:11'!
initialize
	ID := 42! !

!Task class methodsFor: 'private' stamp: 'hk 8/20/2014 18:05'!
nextID
	"must be unique"
	
	^(ID := ID + 1) asString! !

!Task methodsFor: 'testing' stamp: 'hk 7/2/2014 11:57'!
empty
	^stage = #empty! !

!Task methodsFor: 'testing' stamp: 'hk 7/2/2014 12:11'!
finished
	^ stage = #finished! !

!Task methodsFor: 'accessing' stamp: 'hk 7/2/2014 15:59'!
id 
	^id! !

!Task methodsFor: 'accessing' stamp: 'hk 8/20/2014 18:08'!
id: aString
	id := aString
	! !

!Task methodsFor: 'initialize-release' stamp: 'hk 7/2/2014 12:55'!
initialize
	stage := #empty! !

!Task methodsFor: 'accessing' stamp: 'hk 6/30/2014 11:20'!
loadLane 
	^loadLane! !

!Task methodsFor: 'accessing' stamp: 'hk 7/2/2014 12:55'!
loadLane: aString 
	loadLane := aString! !

!Task methodsFor: 'testing' stamp: 'hk 6/30/2014 11:23'!
loaded
	^ stage = #loaded! !

!Task methodsFor: 'simulation' stamp: 'hk 7/7/2014 14:48'!
next    
	self empty ifTrue: [^stage := #loaded ].
	self loaded ifTrue: [^stage := #finished]! !

!Task methodsFor: 'accessing' stamp: 'hk 6/30/2014 11:20'!
unloadLane 
	^unloadLane! !

!Task methodsFor: 'accessing' stamp: 'hk 7/2/2014 12:56'!
unloadLane: aString 
	unloadLane := aString! !
Task initialize!
AGV initialize!
